/**********************************************************************\
**
**                          Copyright (c) 2008-2022 by
**                          HENGSTLER GmbH, Aldingen
**
**      This  software  is furnished under a license and may be used and
**      copied only in accordance with the terms  of  such  license  and
**      with the inclusion of the above    copyright notice.  This software
**      or any other copies thereof may not  be  provided  or  otherwise
**      made  available  to any other person.  No title to and ownership
**      of the software is hereby transferred.
**
**      The  information  in  this software is subject to change without
**      notice and should not be construed as a commitment by Hengstler.
**
**  Module:      Xpm_PrintMonPlugin.h
**  Facility:    S690001 - Hengstler XPM Printer Driver for Windows XP/7/8/10
**  Component:   Print Monitor Plugin (Xpm-PrintMonPlugin)
**  Version:     R8-V1.07-FINAL / 2022OCT18
**  Environment: Windows XP/7/8/10
**  Compiler:    Microsoft Visual Studio 2015
**  Abstract:    Definitions of print monitor plugin interface and registry entries
**  Note:        -
**  History:     R1-V1.00c / 2008JUL11 / HOR / 1st RELEASE - 1st BETA
**               R1-V1.00d / 2008DEC11 / HOR / 1st RELEASE - 2nd BETA
**               R1-V1.00e / 2009MAY05 / HOR / 1st RELEASE - 3rd BETA
**               R1-V1.00f / 2009JUL16 / HOR / 1st RELEASE - 4th BETA
**               R1-V1.00  / 2009AUG27 / HOR / 1st RELEASE - FINAL 
**               ------------------------------------------------------------
**               R2-V1.01a / 2010JAN26 / HOR / 2nd RELEASE - 1st BETA 
**               R2-V1.01b / 2010MAR04 / HOR / 2nd RELEASE - 2nd BETA
**               R2-V1.01c / 2011DEC19 / HOR / M-R2-012-HOR
**                  Complete driver package converted to VS2010 and adapted to Windows 7 32/64 bit
**               R2-V1.01c / 2011DEC19 / HOR / 2nd RELEASE - 3rd BETA
**               R2-V1.01d / 2012JUN27 / HOR / 2nd RELEASE - 4th BETA
**               R2-V1.01e / 2015MAY05 / HOR / 2nd RELEASE - 5th BETA
**               R2-V1.01f / 2016FEB15 / HOR / 2nd RELEASE - 6th BETA
**               R2-V1.01  / 2016AUG19 / HOR / M-R2-041-HOR
**                 Complete driver package converted to VS2015 and adapted to Windows 8/10 32/64 bit
**               R2-V1.01  / 2016AUG19 / HOR / 2nd RELEASE - FINAL
**               ------------------------------------------------------------
**               R3-V1.02  / 2016AUG24 / HOR / M-R3-001-HOR
**                 Additional driver package compilation added for usage within Windows-XP.
**               R3-V1.02  / 2016AUG24 / HOR / 3rd RELEASE - FINAL
**               ------------------------------------------------------------
**               R4-V1.03  / 2018JUL10 / HOR / 4th RELEASE - FINAL
**               ------------------------------------------------------------
**               R5-V1.04  / 2018OCT11 / HOR / 5th RELEASE - FINAL
**               ------------------------------------------------------------
**               R6-V1.05  / 2018DEC07 / HOR / 6th RELEASE - FINAL
**               ------------------------------------------------------------
**               R7-V1.06  / 2022FEB10 / HOR / 7th RELEASE - FINAL
**               ------------------------------------------------------------
**               R8-V1.07  / 2022OCT18 / HOR / 8th RELEASE - FINAL
**               ------------------------------------------------------------
**
\**********************************************************************/

/*
** COMPILER
*/

#pragma once

/*
** INCLUDES
*/ 


#include "..\..\Libraries\Xpm_Api.h"
/*
** MACROS
*/ 

//-- Windows registry key in printer data where print monitor plugin info is stored
#define D_XPM_PMP_REG_KEY                        "XPM-DRIVER"

//-- Windows registry value for corresponding print monitor plugin info
#define D_XPM_PMP_REG_VALUE_PMP_PRINTER_INFO     "XPM-PMP-PRINTER-INFO"
#define D_XPM_PMP_REG_VALUE_PMP_PRINTER_STATUS   "XPM-PMP-PRINTER-STATUS"
#define D_XPM_PMP_REG_VALUE_PMP_PRINTER_SENSOR   "XPM-PMP-PRINTER-SENSOR"
#define D_XPM_PMP_REG_VALUE_PMP_PRINTER_PRESENT  "XPM-PMP-PRINTER_PRESENT"
#define D_XPM_PMP_REG_VALUE_PMP_PRINTER_FEED     "XPM-PMP-PRINTER_FEED"
#define D_XPM_PMP_REG_VALUE_PMP_JOB_INFO         "XPM-PMP-JOB-INFO"
#define D_XPM_PMP_REG_VALUE_PMP_SETTINGS         "XPM-PMP-SETTINGS"

/*
** TYPES
*/ 
   
//********************************************************************************************************
//** TYPE:        TS_XPM_PMP_JOB_INFO
//** ABSTRACT:    Type definition for print monitor plugin structure containing job information 
//********************************************************************************************************

#pragma pack(1)
typedef struct
   { DWORD                         dwSize;                // Size of this structure
     unsigned char                 ubRevision;            // Revision number (D_XPM_PMP_JI_REVISION)
     DWORD                         dwTsLastUpdate;        // Timestamp of last structure update (from GetTickCount)
     DWORD                         dwJobId;               // Identifier of current/last job (-1 indicates no job)
     DWORD                         dwJobStatus;           // Job status (see D_XPM_PMP_JI_JS_...) (if dwJobId != -1)
     DWORD                         dwPageCount;           // Number of pages printed (if dwJobId != -1)
     BYTE                          aubReserved[32];       // Reserved bytes for future use
   } TS_XPM_PMP_JOB_INFO, * PS_XPM_PMP_JOB_INFO;          // Print monitor plugin job information 
#define D_XPM_PMP_JI_REVISION                    0        // Structure revision number for TS_XPM_PMP_JOB_INFO
#define D_XPM_PMP_JI_JS_JOB_ACTIVE               0x0001   // Job status in TS_XPM_PMP_JOB_INFO->dwJobStatus
#define D_XPM_PMP_JI_JS_JOB_FINISHED             0x0002   // Job status in TS_XPM_PMP_JOB_INFO->dwJobStatus
#define D_XPM_PMP_JI_JS_JOB_CANCELLED            0x0003   // Job status in TS_XPM_PMP_JOB_INFO->dwJobStatus
#pragma pack()

//********************************************************************************************************
//** TYPE:        TS_XPM_PMP_PRINTER_STATUS
//** ABSTRACT:    Type definition for print monitor plugin structure containing status information 
//********************************************************************************************************

#pragma pack(1)
typedef struct
   { DWORD                          dwSize;                // Size of this structure
     unsigned char                  ubRevision;            // Revision number (D_XPM_PMP_PST_REVISION)
     DWORD                          dwTsLastUpdate;        // Timestamp of last structure update (from GetTickCount)
     DWORD                          dwDataValidFlag;       // Data valid flags (see D_XPM_PMP_PST_DVF_...)
     TS_XPM_API_PCKT_PRINTER_STATUS sPrinterStatus;        // Printer status (command: GS a+[n1]+[n2],n=0x0000)
     BYTE                           aubReserved[32];       // Reserved bytes for future use
   } TS_XPM_PMP_PRINTER_STATUS, * PS_XPM_PMP_PRINTER_STATUS; // Print monitor plugin printer status 
#define D_XPM_PMP_PST_REVISION                    0        // Structure revision number for TS_XPM_PMP_PRINTER_STATUS
#define D_XPM_PMP_PST_DVF_PRINTER_STATUS          0x0001   // Valid flag, bit offset in TS_XPM_PMP_PRINTER_STATUS->dwDataValidFlag
#pragma pack()

//********************************************************************************************************
//** TYPE:        TS_XPM_PMP_PRINTER_INFO
//** ABSTRACT:    Type definition for print monitor plugin structure containing printer information 
//********************************************************************************************************

#pragma pack(1)
typedef struct
   { DWORD                          dwSize;                // Size of this structure
     unsigned char                  ubRevision;            // Revision number (D_XPM_PMP_PIN_REVISION)
     DWORD                          dwTsLastUpdate;        // Timestamp of last structure update (from GetTickCount)
     DWORD                          dwDataValidFlag;       // Data valid flags (see D_XPM_PMP_PIN_DVF_...)
     TS_XPM_API_PCKT_PRINTER_INFO   sPrinterInfo;          // Printer info (command: GS a+[n1]+[n2],n=0x0001)
     BYTE                           aubReserved[32];       // Reserved bytes for future use
   } TS_XPM_PMP_PRINTER_INFO, * PS_XPM_PMP_PRINTER_INFO;   // Print monitor plugin printer info 
#define D_XPM_PMP_PIN_REVISION                   1         // Structure revision number for TS_XPM_PMP_PRINTER_INFO
#define D_XPM_PMP_PIN_DVF_PRINTER_INFO           0x0001    // Valid flag, bit offset in TS_XPM_PMP_PRINTER_INFO->dwDataValidFlag
#pragma pack()

//********************************************************************************************************
//** TYPE:        TS_XPM_PMP_PRINTER_SENSOR
//** ABSTRACT:    Type definition for print monitor plugin structure containing sensor information 
//********************************************************************************************************

#pragma pack(1)
typedef struct
   { DWORD                          dwSize;                // Size of this structure
     unsigned char                  ubRevision;            // Revision number (D_XPM_PMP_PSE_REVISION)
     DWORD                          dwTsLastUpdate;        // Timestamp of last structure update (from GetTickCount)
     DWORD                          dwDataValidFlag;       // Data valid flags (see D_XPM_PMP_PSE_DVF_...)
     TS_XPM_API_PCKT_PRINTER_SENSOR sPrinterSensor;        // Printer sensor (command: GS a+[n1]+[n2],n=0x0002)
     BYTE                           aubReserved[32];       // Reserved bytes for future use
   } TS_XPM_PMP_PRINTER_SENSOR, * PS_XPM_PMP_PRINTER_SENSOR;  // Print monitor plugin printer sensor
#define D_XPM_PMP_PSE_REVISION                    0        // Structure revision number for TS_XPM_PMP_PRINTER_SENSOR
#define D_XPM_PMP_PSE_DVF_PRINTER_SENSOR          0x0001   // Valid flag, bit offset in TS_XPM_PMP_PRINTER_SENSOR->dwDataValidFlag
#pragma pack()

//********************************************************************************************************
//** TYPE:        TS_XPM_PMP_PRINTER_PRESENT
//** ABSTRACT:    Type definition for print monitor plugin structure containing printer presenter 
//********************************************************************************************************

#pragma pack(1)
typedef struct
   { DWORD                           dwSize;               // Size of this structure
     unsigned char                   ubRevision;           // Revision number (D_XPM_PMP_PPR_REVISION)
     DWORD                           dwTsLastUpdate;       // Timestamp of last structure update (from GetTickCount)
     DWORD                           dwDataValidFlag;      // Data valid flags (see D_XPM_PMP_PPR_DVF_...)
     TS_XPM_API_PCKT_PRINTER_PRESENT sPrinterPresent;      // Printer present (command: GS a+[n1]+[n2],n=0x0040)
     BYTE                            aubReserved[32];      // Reserved bytes for future use
   } TS_XPM_PMP_PRINTER_PRESENT, * PS_XPM_PMP_PRINTER_PRESENT; // Print monitor plugin printer present
#define D_XPM_PMP_PPR_REVISION                   0         // Structure revision number for TS_XPM_PMP_PRINTER_PRESENT
#define D_XPM_PMP_PPR_DVF_PRINTER_PRESENT        0x0001    // Valid flag, bit offset in TS_XPM_PMP_PRINTER_PRESENT->dwDataValidFlag
#pragma pack()

//********************************************************************************************************
//** TYPE:        TS_XPM_PMP_PRINTER_FEED
//** ABSTRACT:    Type definition for print monitor plugin structure containing printer feed
//********************************************************************************************************

#pragma pack(1)
typedef struct
   { DWORD                           dwSize;               // Size of this structure
     unsigned char                   ubRevision;           // Revision number (D_XPM_PMP_PFE_REVISION)
     DWORD                           dwTsLastUpdate;       // Timestamp of last structure update (from GetTickCount)
     DWORD                           dwDataValidFlag;      // Data valid flags (see D_XPM_PMP_PFE_DVF_...)
     TS_XPM_API_PCKT_PRINTER_FEED    sPrinterFeed;         // Printer present (command: GS a+[n1]+[n2],n=0x0010)
     BYTE                            aubReserved[32];      // Reserved bytes for future use
   } TS_XPM_PMP_PRINTER_FEED, * PS_XPM_PMP_PRINTER_FEED;   // Print monitor plugin printer feed
#define D_XPM_PMP_PFE_REVISION                   0         // Structure revision number for TS_XPM_PMP_PRINTER_FEED
#define D_XPM_PMP_PFE_DVF_PRINTER_FEED           0x0001    // Valid flag, bit offset in TS_XPM_PMP_PRINTER_FEED->dwDataValidFlag
#pragma pack()

//********************************************************************************************************
//** TYPE:        TS_XPM_PMP_SETTINGS
//** ABSTRACT:    Type definition for print monitor plugin structure containing settings for the plugin 
//********************************************************************************************************

#pragma pack(1)
typedef struct
   { DWORD                         dwSize;                 // Size of this structure
     unsigned char                 ubRevision;             // Revision number (D_XPM_PMP_SET_REVISION)
     WORD                          wServiceIntervalSec;    // Service interval in sec (30 by default, 0 is disable, 5 is minimum)
     WORD                          wMaxDurationOfPrintSec; // Maximum duration of a single page printout until it is cancelled
     WORD                          wDelayInquiryStartMs;   // Delay at start of inquiry in MS, set to 50 if zero, domain [1..1000]
     WORD                          wDelayInquiryResponseMs;// Delay during inquiry (between write and read) in MS, set to 150 if zero, domain [1..1000]
     WORD                          wDelayInquiryEndMs;     // Delay at end of inquiry in MS, set to 50 if zero, domain [1..1000]
     DWORD                         dwOptions;              // Plugin option bitmask (see D_XPM_PMP_SET_OPT_INQ_...)
     BYTE                          aubReserved[32];        // Reserved bytes for future use
   } TS_XPM_PMP_SETTINGS, * PS_XPM_PMP_SETTINGS;           // Print monitor plugin settings
#define D_XPM_PMP_SET_REVISION                 1           // Structure revision number for TS_XPM_PMP_SETTINGS
#define D_XPM_PMP_SET_OPT_UPDATE_PRINTER_INFO_AT_END_OF_DOC_DISABLED     0x00000001   // Option, disables printer info    inquiry at end of document
#define D_XPM_PMP_SET_OPT_UPDATE_PRINTER_STATUS_AT_END_OF_DOC_DISABLED   0x00000002   // Option, disables printer status  inquiry at end of document
#define D_XPM_PMP_SET_OPT_UPDATE_PRINTER_SENSOR_AT_END_OF_DOC_DISABLED   0x00000004   // Option, disables printer sensor  inquiry at end of document
#define D_XPM_PMP_SET_OPT_UPDATE_PRINTER_PRESENT_AT_END_OF_DOC_DISABLED  0x00000008   // Option, disables printer present inquiry at end of document
#define D_XPM_PMP_SET_OPT_UPDATE_PRINTER_FEED_AT_END_OF_DOC_DISABLED     0x00000010   // Option, disables printer feed    inquiry at end of document
#define D_XPM_PMP_SET_OPT_UPDATE_PRINTER_INFO_DURING_SERVICE_DISABLED    0x00000100   // Option, disables printer info    inquiry during service
#define D_XPM_PMP_SET_OPT_UPDATE_PRINTER_STATUS_DURING_SERVICE_DISABLED  0x00000200   // Option, disables printer status  inquiry during service
#define D_XPM_PMP_SET_OPT_UPDATE_PRINTER_SENSOR_DURING_SERVICE_DISABLED  0x00000400   // Option, disables printer sensor  inquiry during service
#define D_XPM_PMP_SET_OPT_UPDATE_PRINTER_PRESENT_DURING_SERVICE_DISABLED 0x00000800   // Option, disables printer present inquiry during service
#define D_XPM_PMP_SET_OPT_UPDATE_PRINTER_FEED_DURING_SERVICE_DISABLED    0x00001000   // Option, disables printer feed    inquiry during service
#define D_XPM_PMP_SET_OPT_UPDATE_PRINTER_INFO_AT_END_OF_PAGE_DISABLED    0x00010000   // Option, disables printer info    inquiry at end of page
#define D_XPM_PMP_SET_OPT_UPDATE_PRINTER_STATUS_AT_END_OF_PAGE_DISABLED  0x00020000   // Option, disables printer status  inquiry at end of page
#define D_XPM_PMP_SET_OPT_UPDATE_PRINTER_SENSOR_AT_END_OF_PAGE_DISABLED  0x00040000   // Option, disables printer sensor  inquiry at end of page
#define D_XPM_PMP_SET_OPT_UPDATE_PRINTER_PRESENT_AT_END_OF_PAGE_DISABLED 0x00080000   // Option, disables printer present inquiry at end of page
#define D_XPM_PMP_SET_OPT_UPDATE_PRINTER_FEED_AT_END_OF_PAGE_DISABLED    0x00100000   // Option, disables printer feed    inquiry at end of page
#define D_XPM_PMP_SET_OPT_MONITOR_PRESENTER_AT_END_OF_PAGE_DISABLED      0x00200000   // Option, disables presenter monitoring    at end of page
#pragma pack()

/*
** FUNCTIONS
*/ 

//********************************************************************************************************
//** TYPE:        TS_XPM_Pmp_StartMonitor 
//** ABSTRACT:    Type definition for the print monitor plugin function that is called at start of print monitoring
//**              The function must initialize the plugin dll
//**              The function is called after port has been opened 
//** PARAMETER:   pszPortName           Name of the port
//**              pszPrinterName        Name of the printer
//**              hPmCallback           Handle for port monitor callback (read/write port)
//**              pdwServiceIntervalSec Required service interval in sec (min 30, disabled 0)
//**              pfnWritePort          Callback function for writing data to port
//**              pfnReadPort           Callback function for reading data from port
//** RETURN:      hData                 Handle pointing to private plugin data (used in other TF_Xpm_Pmp_... functions)
//** NOTE:        The plugin function is called if available in Xpm_PrintMonPlugin.dll 
//********************************************************************************************************

typedef HANDLE (* TF_Xpm_Pmp_StartMonitor)( LPTSTR pszPortName, LPTSTR pszPrinterName
                                          , HANDLE hPmCallback, DWORD * pdwServiceIntervalSec
                                          , BOOL (*pfnWritePort) (HANDLE, LPBYTE, DWORD, LPDWORD, DWORD)
                                          , BOOL (*pfnReadPort)  (HANDLE, LPBYTE, DWORD, LPDWORD, DWORD)
                                          , VOID (*Lo_PmpCancelDocument) (HANDLE)
                                          );

//********************************************************************************************************
//** TYPE:        TS_XPM_Pmp_EndMonitor 
//** ABSTRACT:    Type definition for the print monitor plugin function that is called at end of print monitoring 
//**              The function must free all allocated memory
//**              The function is called just before port closes 
//** PARAMETER:   hData                 Handle pointing to private plugin data (from TF_Xpm_Pmp_StartMonitor)
//** RETURN:      fResult               Flag indicating success if TRUE and FALSE otherwise
//** NOTE:        The plugin function is called if available in Xpm_PrintMonPlugin.dll
//********************************************************************************************************

typedef BOOL (* TF_Xpm_Pmp_EndMonitor)    (HANDLE hData);

//********************************************************************************************************
//** TYPE:        TS_XPM_Pmp_StartDocument 
//** ABSTRACT:    Type definition for the print monitor plugin function that is called when document printing has started
//** PARAMETER:   hData                 Handle pointing to private plugin data (from TF_Xpm_Pmp_StartMonitor)
//**              pszPrinterName        Name of the printer
//**              dwDocInfoLevel        Level of pbDocInfo indicating DOC_INFO_1/2/3 
//**              pbDocInfo             Pointer to DOC_INFO_1/2/3 structure
//**              dwJobId               Id of job that is being done
//** RETURN:      fResult               Flag indicating success if TRUE and FALSE otherwise
//** NOTE:        The plugin function is called if available in Xpm_PrintMonPlugin.dll
//********************************************************************************************************

typedef BOOL (* TF_Xpm_Pmp_StartDocument)  (HANDLE hData, LPTSTR pszPrinterName, DWORD dwJobId, DWORD dwDocInfoLevel, LPBYTE  pbDocInfo);

//********************************************************************************************************
//** TYPE:        TS_XPM_Pmp_CancelDocument  
//** ABSTRACT:    Type definition for the print monitor plugin function that is called when document printing has an error
//** PARAMETER:   hData                 Handle pointing to private plugin data (from TF_Xpm_Pmp_StartMonitor)
//**              dwDurationOfPrintSec  Number of seconds elapsed since the printout of the page has been started
//** RETURN:      fResult               Flag indicating continue print if FALSE and cancel print otherwise
//** NOTE:        The plugin function is called if available in Xpm_PrintMonPlugin.dll
//********************************************************************************************************

typedef BOOL (* TF_Xpm_Pmp_CancelDocument) (HANDLE hData, DWORD dwDurationOfPrintSec);

//********************************************************************************************************
//** TYPE:        TS_XPM_Pmp_EndDocument  
//** ABSTRACT:    Type definition for the print monitor plugin function that is called before document printing has ended 
//** PARAMETER:   hData                 Handle pointing to private plugin data (from TF_Xpm_Pmp_StartMonitor)
//** RETURN:      fResult               Flag indicating success if TRUE and FALSE otherwise
//** NOTE:        The plugin function is called if available in Xpm_PrintMonPlugin.dll
//********************************************************************************************************

typedef BOOL (* TF_Xpm_Pmp_EndDocument) (HANDLE hData);

//********************************************************************************************************
//** TYPE:        TS_XPM_Pmp_StartPage  
//** ABSTRACT:    Type definition for the print monitor plugin function that is called before page printing has started
//** PARAMETER:   hData                 Handle pointing to private plugin data (from TF_Xpm_Pmp_StartMonitor)
//** RETURN:      fResult               Flag indicating success if TRUE and FALSE otherwise
//** NOTE:        The plugin function is called if available in Xpm_PrintMonPlugin.dll
//********************************************************************************************************

typedef BOOL (* TF_Xpm_Pmp_StartPage) (HANDLE hData);

//********************************************************************************************************
//** TYPE:        TS_XPM_Pmp_EndPage  
//** ABSTRACT:    Type definition for the print monitor plugin function that is called before page printing has completed
//** PARAMETER:   hData                 Handle pointing to private plugin data (from TF_Xpm_Pmp_StartMonitor)
//** RETURN:      fResult               Flag indicating success if TRUE and FALSE otherwise
//** NOTE:        The plugin function is called if available in Xpm_PrintMonPlugin.dll
//********************************************************************************************************

typedef BOOL (* TF_Xpm_Pmp_EndPage) (HANDLE hData);

//********************************************************************************************************
//** TYPE:        TS_XPM_Pmp_ServiceRoutine
//** ABSTRACT:    Type definition for the print monitor plugin function that services periodical calls
//** PARAMETER:   hData                 Handle pointing to private plugin data (from TF_Xpm_Pmp_StartMonitor)
//** RETURN:      fResult               Flag indicating success if TRUE and FALSE otherwise
//** NOTE:        The plugin function is called if available in Xpm_PrintMonPlugin.dll
//********************************************************************************************************

typedef BOOL (* TF_Xpm_Pmp_ServiceRoutine) (HANDLE hData);

/*
** VARIABLES
*/ 
